
#include "bitmap.h"

#include <vector>
#include <string>
#include <fstream>

#include <iostream>

void Bitmap::clear()
{
  bitmap.clear();
}

void Bitmap::addLine(int line, int size)
{
  // Automatic sizing
  if (size == 0)
  {
    size = (bitmap.empty() ? 32 : bitmap[0].size());
  }

  std::vector<bool> vector_line;

  for (int mask = 1 << (size - 1); mask > 0; mask >>= 1)
  {
    vector_line.push_back(line & mask);
  }

  addLine(vector_line);
}

void Bitmap::addLine(std::vector<bool> line)
{
  // Fit line to size if needed…
  if (!bitmap.empty())
  {
    while (line.size() < bitmap[0].size())
    {
      line.push_back(false);
    }
    while (line.size() > bitmap[0].size())
    {
      line.pop_back();
    }
  }
  
  // Add line
  bitmap.push_back(line);
}

void Bitmap::setRow(int state, int index, int size)
{
  if (index < 0 or index >= bitmap.size()) { return; }

  // Automatic sizing
  if (size == 0)
  {
    size = (bitmap.empty() ? 32 : bitmap[0].size());
  }

  std::vector<bool> row(size);

  for (int i = 0, mask = 1 << (size - 1); mask > 0; ++i, mask >>= 1)
  {
    row[i] = (state & mask);
  }

  setRow(row, index);
}

void Bitmap::setRow(std::vector<bool> state, int index)
{
  if (index < 0 or index >= bitmap.size()) { return; }

  if (state.size() != bitmap[0].size())
  {
    state.resize(bitmap[0].size(), false);
  }

  bitmap[index] = state;
}



void Bitmap::setSize(int width, int height)
{
  setHeight(height);
  setWidth(width);
}


void Bitmap::setWidth(int width)
{
  for (std::vector<std::vector<bool> >::iterator row = bitmap.begin();
      row != bitmap.end(); ++row)
  {
    row->resize(width, false);
  }
}


void Bitmap::setHeight(int height)
{
  int width = 0;
  if (bitmap.size()) { width = bitmap[0].size(); }

  std::vector<bool> row_template(width, false);
  bitmap.resize(height, row_template);
}



std::vector<bool> Bitmap::getRow(int index)
{
  if (index < 0 or index >= bitmap.size())
  {
    std::vector<bool> empty;
    return empty;
  }

  return bitmap[index];
}


std::vector<bool> Bitmap::getColumn(int index)
{
  std::vector<bool> result;

  if (bitmap.empty() or index < 0 or index >= bitmap[0].size())
  {
    return result;
  }

  for (std::vector<std::vector<bool> >::iterator row = bitmap.begin();
      row != bitmap.end(); ++row)
  {
    result.push_back((*row)[index]);
  }

  return result;
}


void Bitmap::writePBM(std::string filename)
{
  int height = bitmap.size();
  if (!height) return;
  int width = bitmap[0].size();
  if (!width) return;

  std::ofstream file;
  file.open(filename.c_str());

  file << "P1" << std::endl;
  file << width << " " << height << std::endl;

  for (std::vector<std::vector<bool> >::iterator row = bitmap.begin();
      row != bitmap.end(); ++row)
  {
    for (std::vector<bool>::iterator cell = row->begin();
        cell != row->end(); ++cell)
    {
      file << *cell << " ";
    }
    file << std::endl;
  }

  file.close();
}

void Bitmap::print()
{
  for (std::vector<std::vector<bool> >::iterator row = bitmap.begin();
      row != bitmap.end(); ++row)
  {
    std::cout << "|";
    for (std::vector<bool>::iterator cell = row->begin();
        cell != row->end(); ++cell)
    {
      std::cout << (*cell ? "*" : " ");
    }
    std::cout << "|" << std::endl;
  }
}

std::vector<std::vector<bool> > Bitmap::getBitmap()
{
  return bitmap;
}

