
#include "eca.h"
#include "bitmap.h"
#include "graymap.h"
#include "necklace.h"

#include <vector>
#include <set>
#include <iostream>
#include <sstream>
#include <fstream>
#include <cstdlib>

#define CA_MIN_WIDTH 8
#define CA_MAX_WIDTH 8
#define CA_WINDOW_SIZE 6
#define DENSITY_PLOT true

int togglecount(std::vector<bool>);

int main(int argc, char *argv[])
{
  int batch_number = 0;

  if (argc > 1)
  {
    batch_number = std::atoi(argv[1]);
  }

  int automaton_rules[8][11]
      = {{  0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10},
         { 11,  12,  13,  14,  15,  18,  19,  22,  23,  24,  25},
         { 26,  27,  28,  29,  30,  32,  33,  34,  35,  36,  37},
         { 38,  40,  41,  42,  43,  44,  45,  46,  50,  51,  54},
         { 56,  57,  58,  60,  62,  72,  73,  74,  76,  77,  78},
         { 90,  94, 104, 105, 106, 108, 110, 122, 126, 128, 130},
         {132, 134, 136, 138, 140, 142, 146, 150, 152, 154, 156},
         {160, 162, 164, 168, 170, 172, 178, 184, 200, 204, 232}};

  std::cout << "Min. width: " << CA_MIN_WIDTH << std::endl;
  std::cout << "Max. width: " << CA_MAX_WIDTH << std::endl;
  std::cout << "Window size: " << CA_WINDOW_SIZE << std::endl;

  ECA automaton;
  Bitmap spacetime;
  Graymap plot;

  // Run all batches
  for (int batch_number = 0; batch_number < 8; ++batch_number)
  {
  // One rule at a time
  for (int rule_index = 0; rule_index < 11; ++rule_index)
  {
    int eca_rule = automaton_rules[batch_number][rule_index];
    std::cout << eca_rule << ": ";

    // One CA width at a time
    for (int ca_width = CA_MIN_WIDTH; ca_width <= CA_MAX_WIDTH; ++ca_width)
    {
      if (DENSITY_PLOT)
      {
        plot.clear();
        plot.setSize(CA_WINDOW_SIZE, ca_width + 1);
      }
      
      spacetime.setSize(ca_width, CA_WINDOW_SIZE);
      automaton = ECA(eca_rule, ca_width);

      unsigned int state = 0;
 
      // Evolve from all initial states
      unsigned int initial_state = 0;
      while (initial_state = next_necklace(ca_width, initial_state))
      {
        // Initialise CA
        state = initial_state;
        spacetime.setRow(state, 0, ca_width);

        // Generate CA time steps
        for (int i = 1; i < CA_WINDOW_SIZE; ++i)
        {
          state = automaton.next(state);
          spacetime.setRow(state, i);
        }

        // To hold the spectrum
        int distribution[CA_WINDOW_SIZE] = {0};

        // Count toggles for all cells
        for (int i = 0; i < ca_width; ++i)
        {
          int index = togglecount(spacetime.getColumn(i));
          if (index < 0 or index >= CA_WINDOW_SIZE)
          {
            std::cout << "Impossible toggle count: " << index << std::endl;
            continue;
          }
          distribution[index] += 1;
        }

        // Update the density plot
        if (DENSITY_PLOT)
        {
          for (int i = 0; i < CA_WINDOW_SIZE; ++i)
          {
            plot.setPixel(i, ca_width - distribution[i],
                (plot.getPixel(i, ca_width - distribution[i]) ?
                  plot.getPixel(i, ca_width - distribution[i]) - 1 : 0));
          }
        }
      }

      std::cout << ca_width << " ";
      std::cout.flush();

      // Write density plot
      if (DENSITY_PLOT)
      {
        std::stringstream filename;
        filename << "images/density-plot" << eca_rule << "s" << ca_width
          << "w" << CA_WINDOW_SIZE << ".pgm";
        plot.writePGM(filename.str(), true);
      }
    }
  }
  }
  return 0;
}

int togglecount(std::vector<bool> series)
{
  if (series.empty()) { return 0; }
  bool previous = series[0];
  
  int toggles = 0;
  for (std::vector<bool>::iterator it = (series.begin() + 1);
      it < series.end(); ++it)
  {
    if (*it != previous)
    {
      toggles++;
      previous = *it;
    }
  }

  return toggles;
}

