#include <iostream>
#include <string>
#include <sstream>
#include <vector>

#include <stdint.h>
#include <sqlite3.h>
#include <getopt.h>
#include <stdlib.h>

#include "queries.h"

static uint32_t base, casize, range, winsize;
static int32_t mode;

// List rules based on base, casize, range, winsize
#define RULELIST_MODE 0

// List series of differing ca sizes
#define SIZE_MODE 1

// List series of differing window sizes
#define WINDOW_MODE 2

// List values from all rules
#define RULESPACE_MODE 3


#define DEFAULT_BASE 2
#define DEFAULT_CASIZE 16
#define DEFAULT_RANGE 1
#define DEFAULT_WINSIZE 16
#define DEFAULT_MODE RULELIST_MODE


void usage(const char *program_name)
{
  std::cout << "  Usage: " << program_name << " [OPTIONS]" << std::endl;
  std::cout << "" << std::endl;
  std::cout << "  CA options       Default value" << std::endl;
  std::cout << "  --base <int>     " << DEFAULT_BASE << std::endl;
  std::cout << "  --casize <int>   " << DEFAULT_CASIZE << std::endl;
  std::cout << "  --range <int>    " << DEFAULT_RANGE << std::endl;
  std::cout << "  --winsize <int>  " << DEFAULT_WINSIZE << std::endl;
  std::cout << "" << std::endl;
  std::cout << "  Data options     Lists…" << std::endl;
  std::cout << "  --rulelist       rules" << std::endl;
  std::cout << "  --rulespace      spectral diversities over rules" << std::endl;
  std::cout << "  --sizespace      spectral diversities over ca sizes" << std::endl;
  std::cout << "  --windowspace    spectral diversities over window sizes" << std::endl;
  std::cout << "" << std::endl;
  std::cout << "  Other options" << std::endl;
  std::cout << "  --help           display this help message" << std::endl;
  std::cout << "" << std::endl;
}

void print_first(std::vector<std::pair<uint32_t, uint32_t> > data)
{
  for (std::vector<std::pair<uint32_t, uint32_t> >::iterator it
      = data.begin(); it != data.end(); ++it)
  {
    std::cout << it->first << " ";
  }
  std::cout << std::endl;
}

void print_second(std::vector<std::pair<uint32_t, uint32_t> > data)
{
  for (std::vector<std::pair<uint32_t, uint32_t> >::iterator it
      = data.begin(); it != data.end(); ++it)
  {
    std::cout << it->second << " ";
  }
  std::cout << std::endl;
}

int main(int argc, char **argv)
{
  // Defaults
  base = DEFAULT_BASE;
  range = DEFAULT_RANGE;
  casize = DEFAULT_CASIZE;
  winsize = DEFAULT_WINSIZE;
  mode = DEFAULT_MODE;

  // Command line options
  int c;
  while (1)
  {
    static struct option long_options[] =
    {
      {"base",    required_argument, 0, 'b'},
      {"casize",  required_argument, 0, 'c'},
      {"range",   required_argument, 0, 'r'},
      {"winsize", required_argument, 0, 'w'},
      {"help",          no_argument, 0, 'h'},
      {"windowspace",    no_argument, &mode, WINDOW_MODE},
      {"sizespace",    no_argument, &mode, SIZE_MODE},
      {"rulelist",    no_argument, &mode, RULELIST_MODE},
      {"rulespace",    no_argument, &mode, RULESPACE_MODE},
      {0, 0, 0, 0}
    };

    int option_index = 0;
    c = getopt_long (argc, argv, "b:c:r:w:", long_options, &option_index);

    if (c == -1)
      break;

    switch(c)
    {
      case 0:
        break;
      case 'b':
        base = atoi(optarg);
        break;
      case 'c':
        casize = atoi(optarg);
        break;
      case 'r':
        range = atoi(optarg);
        break;
      case 'w':
        winsize = atoi(optarg);
        break;
      default:
        usage(argv[0]);
        exit(0);
        break;
    }
  }


  // Connect to DB
  sqlite3 *db;
  int returncode = sqlite3_open("data.db", &db);
  if (returncode)
  {
    std::cerr << "Can't open database: " << sqlite3_errmsg(db) << std::endl;
  }


  switch(mode)
  {
    case RULELIST_MODE:
      {
      // Fetch rules
      std::vector<uint64_t> rules
          = fetch_rulelist(db, base, casize, range, winsize);

      // Print rules
      for (std::vector<uint64_t>::iterator it
          = rules.begin(); it != rules.end(); ++it)
      {
        std::cout << *it << " ";
      }
      std::cout << std::endl;
      }
      break;

    case SIZE_MODE:
      {
        std::vector<uint64_t> rules
            = fetch_rulelist(db, base, casize, range, winsize);

        for (std::vector<uint64_t>::iterator rule = rules.begin();
            rule != rules.end(); ++rule)
        {
          // Fetch over range of ca sizes
          std::vector<std::pair<uint32_t, uint32_t> > ca_size_data
              = fetch_over_casizes(db, *rule, base, range, winsize);

          // Print sizes
          print_first(ca_size_data);
          
          // Print spectral diversities
          print_second(ca_size_data);
        }
      }
      break;

    case WINDOW_MODE:
      {
        std::vector<uint64_t> rules
            = fetch_rulelist(db, base, casize, range, winsize);
        
        for (std::vector<uint64_t>::iterator rule = rules.begin();
            rule != rules.end(); ++rule)
        {
          // Fetch over range of window sizes
          std::vector<std::pair<uint32_t, uint32_t> > win_size_data
              = fetch_over_winsizes(db, *rule, base, casize, range);
        
          // Print sizes
          print_first(win_size_data);
          
          // Print spectral diversities
          print_second(win_size_data);
        }
      }
      break;

    case RULESPACE_MODE:
      {
      // Test fetch over all available rules
      std::vector<std::pair<uint64_t, uint32_t> > rule_spread_data
          = fetch_over_rules(db, base, casize, range, winsize);

      // Print rules
      for (std::vector<std::pair<uint64_t, uint32_t> >::iterator it
          = rule_spread_data.begin(); it != rule_spread_data.end(); ++it)
      {
        std::cout << it->first << " ";
      }
      std::cout << std::endl;
      // Print spectral diversities
      for (std::vector<std::pair<uint64_t, uint32_t> >::iterator it
          = rule_spread_data.begin(); it != rule_spread_data.end(); ++it)
      {
        std::cout << it->second << " ";
      }
      std::cout << std::endl;
      }
      break;
  }

  // Close DB connection
  sqlite3_close(db);
  return 0;
}

