
#include "queries.h"

#include <stdint.h>
#include <sqlite3.h>
#include <string>
#include <vector>
#include <iostream>
#include <sstream>


// Fetch list of rules with chosen parameters - full version
std::vector<uint64_t> fetch_rulelist
( sqlite3 *db, uint32_t cellstates, uint32_t dimensions, uint32_t size,
  std::string neighbourhood, uint32_t range, std::string boundarycondition,
  std::string transformation, uint32_t windowsize, uint32_t startposition
)
{
  std::vector<uint64_t> rules;
  sqlite3_stmt *ppStmt;
  const char *pzTail;
  int returncode;
  
  // SQL statement
  std::stringstream s_stmt;
  s_stmt << "SELECT rule FROM data WHERE"
    << " cellstates = " << cellstates
    << " AND dimensions = " << dimensions
    << " AND size = " << size
    << " AND neighbourhood = '" << neighbourhood << "'"
    << " AND range = " << range
    << " AND boundarycondition = '" << boundarycondition << "'"
    << " AND transformation = '" << transformation << "'"
    << " AND windowsize = " << windowsize
    << " AND startposition = " << startposition
    << " ORDER BY rule;";
  std::string str_stmt = s_stmt.str();
  const char *c_stmt = str_stmt.c_str();
  
  //std::cout << c_stmt << std::endl;

  // Prepare stmt
  returncode = sqlite3_prepare_v2(db, c_stmt, -1, &ppStmt, &pzTail);
  if (returncode != SQLITE_OK)
  {
    std::cerr << "Error code " << returncode << " preparing statement ``" 
      << c_stmt << "''" << std::endl;
  }
  
  // Fetch data
  while ((returncode = sqlite3_step(ppStmt)) != SQLITE_DONE)
  {
    if (returncode != SQLITE_ROW)
    {
      std::cerr << "Error code " << returncode << " stepping statement ``"
        << c_stmt << "''" << std::endl;
      break;
    }
    rules.push_back(sqlite3_column_int(ppStmt, 0));
  }

  // Finalise stmt
  sqlite3_finalize(ppStmt);

  return rules;
}

// Fetch list of rules with chosen parameters - short version
std::vector<uint64_t> fetch_rulelist(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t range,
    uint32_t windowsize)
{
  return fetch_rulelist( db, cellstates, 1, size, "", range, "cyclic",
      "togglecount", windowsize, 0);
}




// Fetch spectral diversities over varying ca size - full
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_casizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t dimensions,
    std::string neighbourhood, uint32_t range, std::string boundarycondition,
    std::string transformation, uint32_t windowsize, uint32_t startposition)
{
  std::vector<std::pair<uint32_t, uint32_t> > results;
  sqlite3_stmt *ppStmt;
  const char *pzTail;
  int returncode;
  
  // SQL statement
  std::stringstream s_stmt;
  s_stmt << "SELECT size, codomainsize FROM data WHERE"
    << " rule = " << rule
    << " AND cellstates = " << cellstates
    << " AND dimensions = " << dimensions
    << " AND neighbourhood = '" << neighbourhood << "'"
    << " AND range = " << range
    << " AND boundarycondition = '" << boundarycondition << "'"
    << " AND transformation = '" << transformation << "'"
    << " AND windowsize = " << windowsize
    << " AND startposition = " << startposition
    << " ORDER BY size;";
  std::string str_stmt = s_stmt.str();
  const char *c_stmt = str_stmt.c_str();

  // Prepare stmt
  returncode = sqlite3_prepare_v2(db, c_stmt, -1, &ppStmt, &pzTail);
  if (returncode != SQLITE_OK)
  {
    std::cerr << "Error code " << returncode << " preparing statement ``" 
      << c_stmt << "''" << std::endl;
  }
  
  // Fetch data
  while ((returncode = sqlite3_step(ppStmt)) != SQLITE_DONE)
  {
    if (returncode != SQLITE_ROW)
    {
      std::cerr << "Error code " << returncode << " stepping statement ``"
        << c_stmt << "''" << std::endl;
      break;
    }
    results.push_back(std::pair<uint32_t, uint32_t>
        (sqlite3_column_int(ppStmt, 0),
         sqlite3_column_int(ppStmt, 1)));
  }

  // Finalise stmt
  sqlite3_finalize(ppStmt);
  
  return results;
}

// Fetch spectral diversities over varying ca size - short
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_casizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t range,
    uint32_t windowsize)
{
  return fetch_over_casizes( db, rule, cellstates, 1, "", range, "cyclic",
      "togglecount", windowsize, 0);
}



// Fetch spectral diversities over varying window size - full
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_winsizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t size,
    uint32_t dimensions, std::string neighbourhood, uint32_t range,
    std::string boundarycondition, std::string transformation,
    uint32_t startposition)
{
  std::vector<std::pair<uint32_t, uint32_t> > results;
  sqlite3_stmt *ppStmt;
  const char *pzTail;
  int returncode;
  
  // SQL statement
  std::stringstream s_stmt;
  s_stmt << "SELECT windowsize, codomainsize FROM data WHERE"
    << " rule = " << rule
    << " AND cellstates = " << cellstates
    << " AND dimensions = " << dimensions
    << " AND size = " << size
    << " AND neighbourhood = '" << neighbourhood << "'"
    << " AND range = " << range
    << " AND boundarycondition = '" << boundarycondition << "'"
    << " AND transformation = '" << transformation << "'"
    << " AND startposition = " << startposition
    << " ORDER BY windowsize;";
  std::string str_stmt = s_stmt.str();
  const char *c_stmt = str_stmt.c_str();

  // Prepare stmt
  returncode = sqlite3_prepare_v2(db, c_stmt, -1, &ppStmt, &pzTail);
  if (returncode != SQLITE_OK)
  {
    std::cerr << "Error code " << returncode << " preparing statement ``" 
      << c_stmt << "''" << std::endl;
  }
  
  // Fetch data
  while ((returncode = sqlite3_step(ppStmt)) != SQLITE_DONE)
  {
    if (returncode != SQLITE_ROW)
    {
      std::cerr << "Error code " << returncode << " stepping statement ``"
        << c_stmt << "''" << std::endl;
      break;
    }
    results.push_back(std::pair<uint32_t, uint32_t>
        (sqlite3_column_int(ppStmt, 0),
         sqlite3_column_int(ppStmt, 1)));
  }

  // Finalise stmt
  sqlite3_finalize(ppStmt);
  
  return results;

}

// Fetch spectral diversities over varying window size - short
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_winsizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t size,
    uint32_t range)
{
  return fetch_over_winsizes( db, rule, cellstates, size, 1, "", range,
      "cyclic", "togglecount", 0);
}




// Fetch spectral diversities over varying rules - full
std::vector<std::pair<uint64_t, uint32_t> > fetch_over_rules(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t dimensions,
    std::string neighbourhood, uint32_t range, std::string boundarycondition,
    std::string transformation, uint32_t startposition, uint32_t windowsize)
{
  std::vector<std::pair<uint64_t, uint32_t> > results;
  sqlite3_stmt *ppStmt;
  const char *pzTail;
  int returncode;
  
  // SQL statement
  std::stringstream s_stmt;
  s_stmt << "SELECT rule, codomainsize FROM data WHERE"
    << " cellstates = " << cellstates
    << " AND dimensions = " << dimensions
    << " AND size = " << size
    << " AND neighbourhood = '" << neighbourhood << "'"
    << " AND range = " << range
    << " AND boundarycondition = '" << boundarycondition << "'"
    << " AND transformation = '" << transformation << "'"
    << " AND startposition = " << startposition
    << " AND windowsize = " << windowsize
    << " ORDER BY rule;";
  std::string str_stmt = s_stmt.str();
  const char *c_stmt = str_stmt.c_str();

  // Prepare stmt
  returncode = sqlite3_prepare_v2(db, c_stmt, -1, &ppStmt, &pzTail);
  if (returncode != SQLITE_OK)
  {
    std::cerr << "Error code " << returncode << " preparing statement ``" 
      << c_stmt << "''" << std::endl;
  }
  
  // Fetch data
  while ((returncode = sqlite3_step(ppStmt)) != SQLITE_DONE)
  {
    if (returncode != SQLITE_ROW)
    {
      std::cerr << "Error code " << returncode << " stepping statement ``"
        << c_stmt << "''" << std::endl;
      break;
    }
    results.push_back(std::pair<uint32_t, uint32_t>
        (sqlite3_column_int(ppStmt, 0),
         sqlite3_column_int(ppStmt, 1)));
  }

  // Finalise stmt
  sqlite3_finalize(ppStmt);
  
  return results;
}

// Fetch spectral diversities over varying rules - short
std::vector<std::pair<uint64_t, uint32_t> > fetch_over_rules(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t range,
    uint32_t windowsize)
{
  return fetch_over_rules( db, cellstates, size, 1, "", range, "cyclic",
      "togglecount", 0, windowsize);
}

