
#include <iostream>
#include <cstdlib>
#include <string>
#include <sstream>
#include <vector>
#include <fstream>

#define DEFAULT_WIDTH 24
#define DEFAULT_HEIGHT 16
#define DEFAULT_INITIAL_CONDITION "1"

void usage(char *argv0)
{
  std::cerr << "Usage: " << argv0 << " [OPTION]... RULE [RULE]...\n"
            << "Options:\n"
            << "\t-?,--help                                 Show this help message\n"
            << "\t-w,--width WIDTH                          ECA width\n"
            << "\t-h,--height HEIGHT                        Number of time steps\n"
            << "\t-i,--initial_condition INITIAL_CONDITION  Start states (string, [01]+)"
            << std::endl;
}


int main(int argc, char *argv[])
{
  // We should have at least one argument
  if (argc < 2)
  {
    usage(argv[0]);
    return 1;
  }

  // Default values
  int width = DEFAULT_WIDTH;
  int height = DEFAULT_HEIGHT;
  std::string initial_condition(DEFAULT_INITIAL_CONDITION);

  std::vector<int> rules;

  // Read command line arguments
  for (int i = 1; i < argc; ++i)
  {
    std::string arg = argv[i];
    // Help
    if ((arg == "-?") or (arg == "--help"))
    {
      usage(argv[0]); return 0;
    }
    // Width
    else if ((arg == "-w") or (arg == "--width"))
    {
      if (i+1 < argc) { width = std::atoi(argv[++i]); }
      else { std::cerr << "--width option requires one argument." << std::endl; return 1; }
    }
    // Height
    else if ((arg == "-h") or (arg == "--height"))
    {
      if (i+1 < argc) { height = std::atoi(argv[++i]); }
      else { std::cerr << "--height option requires one argument." << std::endl; return 1; }
    }
    // Initial condition
    else if ((arg == "-i") or (arg == "--initial_condition"))
    {
      if (i+1 < argc)
      {
        std::string input = argv[++i];
        for (std::string::iterator it = input.begin(); it != input.end(); ++it)
        {
          if ((*it != '1') and (*it != '0'))
          {
            std::cerr << "--initial_condition option argument must be of the form '[01]+'." << std::endl;
            return 1;
          }
        }
        initial_condition = input;
      }
      else { std::cerr << "--initial_condition option requires one argument." << std::endl; return 1; }
    }
    // Rule
    else
    {
      rules.push_back(std::atoi(argv[i]));
    }
  }

  // CHeck that we actually got any RULEs
  if (rules.empty()) { usage(argv[0]); return 1; }


  // For all rules
  for (std::vector<int>::iterator rule = rules.begin(); rule != rules.end(); ++rule)
  {
    
    // Initialize spacetime plot
    std::vector<std::vector<bool> > spacetime(height, std::vector<bool>(width, false));

    if (initial_condition.size() > width)
    {
      initial_condition.erase(width);
    }
    int padding_front = (width - initial_condition.size()) / 2;
    int padding_back = width - initial_condition.size() - padding_front;

    for (int i = 0; i < width; ++i)
    {
      if ((i < padding_front)
          or (i > (initial_condition.size() + padding_front)))
      {
        spacetime[0][i] = false;
      }
      else
      {
        spacetime[0][i] = (initial_condition[i - padding_front] == '1' ? true : false);
      }
    }

    // Initialize rule table
    bool ruleset[8] = {false};
    int temp = *rule;
    for (int i = 0; i < 8; ++i)
    {
      ruleset[i] = ((temp & 1) ? true : false);
      temp >>= 1;
    }

    // Simulate
    for (int i = 1; i < height; ++i)
    {
      for (int j = 0; j < width; ++j)
      {
        int table_index = (spacetime[i-1][(j+width-1) % width] ? 1 : 0);
        table_index <<= 1;
        table_index |= (spacetime[i-1][j] ? 1 : 0);
        table_index <<= 1;
        table_index |= (spacetime[i-1][(j+1) % width] ? 1 : 0);

        spacetime[i][j] = ruleset[table_index];
      }
    }
    
    // Print to file
    std::stringstream filename;
    filename << "eca" << *rule << "w" << height << "s" << width << ".pbm";

    std::ofstream file;
    file.open(filename.str().c_str());

    file << "P1" << std::endl;
    file << width << " " << height << std::endl;

    for (std::vector<std::vector<bool> >::iterator row = spacetime.begin();
        row != spacetime.end(); ++row)
    {
      for (std::vector<bool>::iterator cell = row->begin(); cell != row->end(); ++cell)
      {
        file << *cell << " ";
      }
      file << std::endl;
    }
    
    file.close();
  }

  return 0;
}

